// This is a modified version of
// https://github.com/hidekatsu-izuno/nashorn-promise/blob/master/index.js
var JCompletableFuture = Java.type('java.util.concurrent.CompletableFuture');
var JCompleteFutureArray = Java.type('java.util.concurrent.CompletableFuture[]');
var JCompletionException = Java.type('java.util.concurrent.CompletionException');
var JThrowable = Java.type('java.lang.Throwable');

var Promise = function(resolver, futures) {
	if (resolver instanceof JCompletableFuture) {
		this._future = resolver;
		this._futures = futures;
	} else {
		var func = Java.synchronized(function() {
			var status, result;

			resolver(function(value) {
				status = 'fulfilled';
				result = value;
			}, function(reason) {
				status = 'rejected';
				result = reason;
			});
			
			while (status === undefined);
				
			if (status === 'fulfilled')
				return { result: result };
			else if (status === 'rejected') {
				throw result;
			}
		}, this);
		
		this._future = JCompletableFuture.supplyAsync(func);
	}
};

Promise.all = function(array) {
	var futures = array.map(function(elem) {
		if (elem instanceof Promise)
			return elem._future;

		return Promise.resolve(elem)._future;
	});

	return new Promise(JCompletableFuture.allOf(Java.to(futures, JCompleteFutureArray)), futures);
};

Promise.race = function(array) {
	var futures = array.map(function(elem) {
		if (elem instanceof Promise)
			return elem._future;
			
		return Promise.resolve(elem)._future;
	});

	return new Promise(JCompletableFuture.anyOf(Java.to(futures, JCompleteFutureArray)));
};

Promise.resolve = function(value) {
	if (value instanceof Promise) {
		return value;
	} else if (value !== null && (typeof value === 'function' || typeof value === 'object') && typeof value.then === 'function') {
		return new Promise(function(fulfill, reject) {
			try {
				return { result: value.then(fulfill, reject) };
			} catch (e) {
				throw new JCompletionException(e);
			}
		});
	} else {
		return new Promise(JCompletableFuture.completedFuture({
			result: value
		}));
	}
};

Promise.reject = function(value) {
	return new Promise(function(_, reject) {
		reject(value);
	});
};

Promise.prototype.then = function(onFulfillment, onRejection) {
	return new Promise(this._future.handle(function(success, error) {
		if (success === null && error === null && this._futures !== null) {
			success = {
				result: this._futures.map(function(elem) {
					return elem.get().result;
				})
			};
		}

		function reject(reason) {
			if (typeof onRejection === 'function') {
				try {
					while (reason instanceof JThrowable) {
						var cause = reason.getCause();
						reason = cause === null ? reason.getMessage() : cause;
					}

					return { result: onRejection(reason) };
				} catch (e) {
					throw new JCompletionException(e)
				}
			}
			throw reason;
		}

		if (success !== null) {
			if (typeof onFulfillment === 'function') {
				try {
					var value = success.result;
					if (value instanceof Promise)
						return { result: onFulfillment(value._future.get().result) };

					return { result: onFulfillment(success.result) };
				} catch (e) {
					reject(e);
				}
			}

			return success;
		} else if (error !== null) {
			try {
				return reject(error);
			} catch (e) {
				throw new JCompletionException(e);
			}
		}
	}.bind(this)));
};

Promise.prototype.catch = function(onRejection) {
	return this.then(null, onRejection);
};